Scriptname SOS_AddonQuest_Script extends Quest  

;----------ADDON PROPERTIES----------

FormList Property SOS_Addon_CompatibleRaces Auto
FormList Property SOS_Addon_RaceSizes Auto
FormList Property SOS_Addon_RaceProbabilities Auto
FormList Property SOS_Addon_Schlonged Auto

Faction Property SOS_Addon_Faction Auto
Armor Property SOS_Addon_Genitals Auto
Potion Property SOS_Addon_Potion Auto
GlobalVariable Property SOS_Addon_Gender Auto

GlobalVariable Property SOS_Addon_Bone_GenBase Auto
GlobalVariable Property SOS_Addon_Bone_GenScrotum Auto
GlobalVariable Property SOS_Addon_Bone_Gen01 Auto
GlobalVariable Property SOS_Addon_Bone_Gen02 Auto
GlobalVariable Property SOS_Addon_Bone_Gen03 Auto
GlobalVariable Property SOS_Addon_Bone_Gen04 Auto
GlobalVariable Property SOS_Addon_Bone_Gen05 Auto
GlobalVariable Property SOS_Addon_Bone_Gen06 Auto

;----------SOS CORE PROPERTIES----------

FormList Property SOS_Addons Auto

;----------EVENTS----------

Event OnInit()
	If !SelfCheck()
		Debug.MessageBox("SOS Addon '" + self.GetName() + "' is dirty, plese notify the Addon author to fix it.\nIn order to prevent data corruption it was not loaded!") ; todo translate
		Return
	EndIf

	RegisterAddon()
EndEvent

Function RegisterAddon()
	
	; initialize all compatible races as enabled
	int i = 0
	int raceCount = SOS_Addon_CompatibleRaces.GetSize()
	while(i < raceCount)
		Form aRace = SOS_Addon_CompatibleRaces.GetAt(i)
		StorageUtil.FormListAdd(Self, "SOS_CompatibleRaces", aRace, allowDuplicate = false)
		StorageUtil.FormListAdd(Self, "SOS_EnabledRaces", aRace, allowDuplicate = false)
		StorageUtil.FormListAdd(None, "SOS_KnownCompatibleRaces", aRace, allowDuplicate = false)
		StorageUtil.IntListAdd(Self, "SOS_RaceSizes", (SOS_Addon_RaceSizes.GetAt(i) as GlobalVariable).GetValueInt())
		StorageUtil.FloatListAdd(Self, "SOS_RaceProbabilities", (SOS_Addon_RaceProbabilities.GetAt(i) as GlobalVariable).GetValue())
		i += 1
	endwhile
	
	StorageUtil.FormListAdd(Self, "SOS_Factions", SOS_Addon_Faction, allowDuplicate = false)
	StorageUtil.SetIntValue(Self, "SOS_Genders", SOS_Addon_Gender.GetValueInt())
	StorageUtil.FormListAdd(Self, "SOS_GenitalArmors", SOS_Addon_Genitals, allowDuplicate = false)
	
	StorageUtil.FormListAdd(None, "SOS_Addons", Self, allowDuplicate = false)
	SOS_Addons.AddForm(Self)

	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_GenBase.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_GenScrotum.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen01.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen02.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen03.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen04.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen05.GetValue(), true)
	StorageUtil.FloatListAdd(Self, "SOS_Bones", SOS_Addon_Bone_Gen06.GetValue(), true)

	String addonName = StringUtil.Substring(SOS_Addon_Faction.GetName(), 4)
	Debug.Trace("SOS: Addon " + addonName + " registered.")
	Debug.Notification("SOS Addon " + addonName + " loaded!") ; todo translate
	
	; You can't expect every SOS addon author to fill a new property in their plugin, so we have to retrieve SOS_SetupQuest this way.
	(Game.GetFormFromFile(0xD62, "Schlongs of Skyrim.esp") as SOS_SetupQuest_Script).AddonsRegistered = True

EndFunction

bool Function SelfCheck()
	{Ensures that the mod author did not forget to set any property in the CK which could lead to abortion midway during initialization leaving the formlists messed up and therefore breaking all other addons}
	
	String addonName = StringUtil.Substring(SOS_Addon_Faction.GetName(), 4)
	
	; addon preference collections
	if(!(SOS_Addon_CompatibleRaces && SOS_Addon_EnabledRaces && SOS_Addon_RaceSizes && SOS_Addon_RaceProbabilities))
		Debug.Trace("SOS: Dirty addon '" + addonName + "' - settings collection references missing or invalid.")
		return false
	else
		int raceCount = SOS_Addon_CompatibleRaces.GetSize()
		if(raceCount < 1 || SOS_Addon_EnabledRaces.GetSize() > 0 || SOS_Addon_RaceSizes.GetSize() != raceCount || SOS_Addon_RaceProbabilities.GetSize() != raceCount)
			Debug.Trace("SOS: Dirty addon '" + addonName + "' - settings collection content (" + raceCount + ", " + SOS_Addon_EnabledRaces.GetSize() + ", " + SOS_Addon_RaceSizes.GetSize() + "," + SOS_Addon_RaceProbabilities.GetSize() + ")")
			return false
		endif
	endif
	
	; addon assets
	if(!(SOS_Addon_Faction && SOS_Addon_Genitals && SOS_Addon_Potion && SOS_Addon_Gender && SOS_Addon_Schlonged))
		Debug.Trace("SOS: Dirty addon '" + addonName + "' - asset references missing or invalid.")
		return false
	endif

	; addon subshape globals
	if(!(SOS_Addon_Bone_GenBase && SOS_Addon_Bone_GenScrotum && SOS_Addon_Bone_Gen01 && SOS_Addon_Bone_Gen02 && SOS_Addon_Bone_Gen03 && SOS_Addon_Bone_Gen04 && SOS_Addon_Bone_Gen05 && SOS_Addon_Bone_Gen06))
		Debug.Trace("SOS: Dirty addon '" + addonName + "' - subshape global references missing or invalid.")
		return false
	endif
	
	; sos core esm references
	If !SOS_Addons
		Debug.Trace("SOS: Dirty addon '" + addonName + "' - core esm references missing or invalid.")
		Return False
	EndIf
	
	return true
EndFunction

Function UnregisterAddon()
	{Tries to remove itself from everywhere it has put itself durint the init}
	String addonName = StringUtil.Substring(SOS_Addon_Faction.GetName(), 4)
	
	Debug.Trace("SOS: Trying to remove everything added by the " + addonName + " addon...")
	
	StorageUtil.FormListClear(self, "SOS_CompatibleRaces")
	StorageUtil.FormListClear(self, "SOS_EnabledRaces")
	StorageUtil.IntListClear(self, "SOS_RaceSizes")
	StorageUtil.FloatListClear(self, "SOS_RaceProbabilities")
	StorageUtil.FormListClear(self, "SOS_Factions")
	StorageUtil.UnsetIntValue(self, "SOS_Genders")
	StorageUtil.FormListClear(self, "SOS_Schlonged")
	StorageUtil.FormListClear(self, "SOS_GenitalArmors")

	StorageUtil.FormListRemove(None, "SOS_Addons", self, allInstances = true)
	SOS_Addons.RemoveAddedForm(self)
	
	Debug.Trace("SOS: Unregistration of " + addonName + " addon hopefully complete.")
	Debug.Notification("SOS Addon " + addonName + " unregistered!") ; todo translate
EndFunction

; the following two functions are no longer used, but kept here for compatibility:

String Function GetAddonName()
	; by convention the name of the Faction is unique and describes the schlong type as short and precise as possible, prefixed with "SOS "
	Return StringUtil.Substring(SOS_Addon_Faction.GetName(), 4)
EndFunction

String Function GetName()
	Return GetAddonName()
EndFunction

; DEPRECATED PROPERTIES

FormList Property SOS_AddonLock Auto
FormList Property SOS_CompatibleRaces Auto
FormList Property SOS_EnabledRaces Auto
FormList Property SOS_RaceSizes Auto
FormList Property SOS_RaceProbabilities Auto
FormList Property SOS_Factions Auto
FormList Property SOS_GenitalArmors Auto
FormList Property SOS_Genders Auto
FormList Property SOS_ShapePotions Auto
FormList Property SOS_Bone_GenBase Auto
FormList Property SOS_Bone_GenScrotum Auto
FormList Property SOS_Bone_Gen01 Auto
FormList Property SOS_Bone_Gen02 Auto
FormList Property SOS_Bone_Gen03 Auto
FormList Property SOS_Bone_Gen04 Auto
FormList Property SOS_Bone_Gen05 Auto
FormList Property SOS_Bone_Gen06 Auto
FormList Property SOS_Schlonged Auto
LeveledItem Property SOS_LItemShapePotions Auto
FormList Property SOS_Addon_EnabledRaces Auto
