Scriptname SOS_Maintenance extends Quest

SOS_SetupQuest_Script Property SOS Auto
SOS_PlayerAliasScript Property PlayerAliasScript Auto
Bool firstTime

Event OnInit()
	Init()
EndEvent

Function Init()
	If !CheckRequirements()
		Return
	EndIf
	
	int installedVersion = CheckUpdate()
	
	If firstTime
		SOS.Start()
	Else
		PlayerAliasScript.CheckAddons()
	EndIf
	
	SOS.SOS_VersionInt = installedVersion
	
	SOS.Initialize(firstTime)
EndFunction

Int Function CheckUpdate()
	
	Int loadedVersion = SOS.SOS_VersionInt
	Int installedVersion = 300004 ; Current version
	Debug.Trace("SOS Maintenance: loaded version is " + loadedVersion)
	firstTime = False
	
	If loadedVersion < installedVersion
	
		If loadedVersion
		
			Debug.Trace("SOS Maintenance: updating to version 3.00.005") ; Current version

			If loadedVersion < 300000
				StorageUtil.FormListClear(None, "SOS_ConcealingArmors")
				StorageUtil.FormListClear(None, "SOS_RevealingArmors")
			EndIf
			If loadedVersion < 300002
				SOS.config.PerformVersionUpdate(300002)
			EndIf

			; add switch for each subsequent release here
			SOS.Notify("$SOS update complete")

		ElseIf SOS.SOS_Version
			; in 2.04.012 (and 2.05.026) the version number was changed to int to avoid floating point issues
			; fix version
			SOS.SOS_VersionInt = (SOS.SOS_Version * 100000) as Int
			; re-run the update
			Return CheckUpdate()
		Else
			; no version saved
			firstTime = True
			
		Endif

	else
		Debug.Trace("SOS Maintenance: Current version " + SOS.SOS_VersionInt + " already installed.")
	endIf
	
	Return installedVersion

EndFunction

Bool Function CheckRequirements()
	int minSkseVersion = 55 ; version 2.0.6
	Int installedSkseVersion = SKSE.GetVersionRelease()
	String minSkseVersionStr = "2.0.6"

	int minPUVersion = 39
	If installedSkseVersion >= 66  ; version 2.1.2 or later for AE
		minPUVersion = 41
	EndIf
	Int installedPUVersion = PapyrusUtil.GetVersion()

	Int minSOSVersion = 300004
	Int installedSOSVersion = -1 ;SKSE.GetPluginVersion("SchlongsOfSkyrim")  ; optional SOS DLL support disabled for now
	
	bool ok = installedSkseVersion >= minSkseVersion
	Debug.Trace("SOS CheckRequirements: SKSE.GetVersionRelease = " + installedSkseVersion)
	If !ok
		Debug.TraceAndBox("SOS Warning!\n\nSKSE not found, or the installed version is too old.\n\nFound " + SKSE.GetVersion() + "." + SKSE.GetVersionMinor()+ "." + SKSE.GetVersionBeta() + ", expected " + minSkseVersionStr + " or higher.\n\nPlease exit the game without saving and install the last SKSE version")
		Return false
	EndIf

	If installedSOSVersion >= minSOSVersion
		Debug.Trace("SOS CheckRequirements: Optional SchlongsOfSkyrim.dll found (version " + installedSOSVersion + ")")
		If !SOS.useDLL
			SOS.InvalidateAllRacesCache(newDLL = True)  ; clean probability caches since they might cause trouble when the dll is uninstalled later
			SOS.useDLL = True
		EndIf
		Return True
	Else
		Debug.Trace("SOS CheckRequirements: No working SchlongsOfSkyrim.dll found. Functions will be emulated in Papyrus.")
		SOS.useDLL = False
	EndIf

	ok = installedPUVersion >= minPUVersion
	Debug.Trace("SOS CheckRequirements: PapyrusUtil.GetVersion = " + installedPUVersion)
	If !ok
		Debug.TraceAndBox("SOS Warning!\n\nPapyrusUtil.dll not found, or the installed version is too old.\n\nFound " + installedPUVersion + ", expected " + minPUVersion + ".\n\nPlease exit the game and reinstall PapyrusUtil.")
		Return false
	EndIf

	Return true
EndFunction
